// $Id: rf_integer.hh 2187 2006-12-15 00:17:56Z orlov $

#ifndef __rf_integer_hh__
#define __rf_integer_hh__

#include "rf_types.hh"
#include "rf_expr.hh"
#include "rf_term.hh"
#include "pxx_string.hh"
#include <wchar.h>

#ifdef RFRT_WITHOUT_MPINT

#include "rf_short_int.hh"

namespace rftype
{

typedef Int32 Integer;

}

#elif defined RFRT_WITHOUT_GMP

#include "rf_mpint.hh"

namespace rftype
{

typedef RF_SYMBOL(MPInt) Integer;

}

#else // !defined RFRT_WITHOUT_GMP && !defined RFRT_WITHOUT_MPINT

#include <gmp.h>

namespace rftype
{

using namespace rfrt;

class Integer :
  public Term
{

  NO_ASSIGN(Integer)

public:

  class MpzContainer
  {
    friend class Integer;
    friend inline Integer operator - (intptr_t _n1, Integer const& _n2) ;

    uintptr_t ref_count ;
    mpz_t     value ;
  };

  static pxx::ChunkAllocator<MpzContainer>* mpz_allocator ;

private:

  inline Integer (MpzContainer* _p) ;

  inline const mpz_t& get_mpz_val () const ;

public:
  ///
  /// Exceptions
  class Error {};
  class InvalidParameter : public Error {};
  class OutOfMemory : public Error {};

  inline Integer (intptr_t _n) ;
  inline Integer (Integer const& _n) ;
  inline Integer (char const* _str) ;
  inline Integer (wchar_t const* _str) ;
  inline Integer (pxx::WString const& _str) ;
  inline ~Integer () ;
//  inline Integer& operator = (Integer const& _n) ;
  inline bool operator == (Integer const& _n) const ;
  inline bool operator == (unsigned long _n) const ;
  inline bool operator == (long _n) const ;
  inline bool operator == (unsigned int _n) const ;
  inline bool operator == (int _n) const ;
  inline int cmp (Integer const& _n) const ;
  inline int cmp (unsigned long _n) const ;
  inline int cmp (long _n) const ;
  inline int cmp (unsigned int _n) const ;
  inline int cmp (int _n) const ;
  inline Integer operator + (Integer const& _n) const ;
  inline Integer operator + (intptr_t _n) const ;
  friend inline Integer operator + (intptr_t _n1, Integer const& _n2) ;
  inline Integer operator - (Integer const& _n) const ;
  inline Integer operator - (intptr_t _n) const ;
  friend inline Integer operator - (intptr_t _n1, Integer const& _n2) ;
  inline Integer operator * (Integer const& _n) const ;
  inline Integer operator * (intptr_t _n) const ;
  friend inline Integer operator * (intptr_t _n1, Integer const& _n2) ;
  inline Integer operator / (Integer const& _n) const ;
  inline Integer operator / (intptr_t _n) const ;
  friend inline Integer operator / (intptr_t _n1, Integer const& _n2) ;
  inline Integer operator % (Integer const& _n) const ;
  inline Integer operator % (intptr_t _n) const ;
  friend inline Integer operator % (intptr_t _n1, Integer const& _n2) ;
  inline Integer div_rem (Integer const& _n, Integer* _remp) const ;
  inline Integer div_rem (intptr_t _n, Integer* _remp) const ;
  friend inline Integer div_rem (
    intptr_t _n1, Integer const& _n2, Integer* _remp) ;
  inline Integer operator - () const ;
  inline int sign () const ;

  static inline Integer gcd (Integer const& _n1, Integer const _n2) ;

  //  inline Integer operator < (unsigned long _n) const ;
  //  inline Integer operator > (unsigned long _n) const ;

  inline long to_int () const ;

  inline operator pxx::WString () const ;

  static inline Expr create_expr (intptr_t _n) ;
  static inline Expr create_expr (char const* _str) ;
  static inline Expr create_expr (wchar_t const* _str) ;
  static inline Expr create_expr (pxx::WString& _str) ;
  static inline Expr create_expr (Integer const& _n) ;

  static inline Expr create_sum  (Integer const& _n1, Integer const& _n2) ;
  static inline Expr create_mult (Integer const& _n1, Integer const& _n2) ;

  inline size_t get_char_len () const ;

  static void ctor (Integer* _to, Integer const* _from) ;
  static void dtor (Integer* _n) ;
  static bool eq (Integer const* _n1, Integer const* _n2) ;
  static int compare (Integer const* _n1, Integer const* _n2) ;
  static uint32_t hash (Integer const* _shint) ;
  static pxx::WString to_string (Integer const* _shint) ;
  static short_type_funcs_t funcs ;
  static TypeRegister reg ;

  ///
  /// Term class for template instantiations
  static const term_class_t term_class = short_term;
};

}

#endif // !defined RFRT_WITHOUT_GMP && !defined RFRT_WITHOUT_MPINT

#endif // __rf_integer_hh__
