#ifndef __rf_types_hh__
#define __rf_types_hh__

#include "rf_common.hh"
#include "pxx_string.hh"

namespace rfrt
{

class Term ;

static const unsigned short_types_count = 0x1000 ;
static const unsigned max_type = 0xFFFF ;

typedef enum
{
  SYM_BORDER = 0,
  type_parenth = 1,
  type_object,
  type_char,
  type_word,
  type_int,
  type_int32,
  type_user,
  type_object_begin = short_types_count,
  type_func,
  type_box,
  type_vector,
  type_string,
  type_table,
  type_channel,
  type_user_object,
  type_expr = max_type
} internal_type_t ;

extern unsigned current_short_type ;
extern unsigned current_object_type ;

typedef struct
{
  unsigned type ;
  ///
  /// Copy constructor for short symbol
  void (*ctor)(Term*, Term const*) ;
  ///
  /// Destructor for short symbol
  void (*dtor)(Term*) ;
  ///
  /// Equality check for short symbol (may be faster than compare)
  bool (*eq)(Term const*, Term const*) ;
  ///
  /// Compare function for short symbol
  int (*compare)(Term const*, Term const*) ;
  ///
  /// Hash function for short symbol.
  uint32_t (*hash)(Term const*) ;
  ///
  /// Conversion to string.
  pxx::WString (*to_string)(Term const*) ;
  ///
  /// If the term has a name then set pointer to it and return its length.
  /// Else return (size_t)-1.
  size_t (*get_name)(Term const*, wchar_t const**) ;
} short_type_funcs_t ;

extern short_type_funcs_t* short_type_funcs[short_types_count] ;

class TypeRegister
{
  unsigned type ;
public:
  inline TypeRegister (short_type_funcs_t* _funcs, unsigned _typenum = 0) ;
  inline TypeRegister (unsigned _type, short_type_funcs_t* _funcs) ;
  inline unsigned get_type () const ;
};

class ObjectRegister
{
  unsigned type ;
public:
  inline ObjectRegister (unsigned _typenum = 0) ;
  inline unsigned get_type () const ;
};

typedef enum
{
  flat_term,
  short_term,
  object_term
} term_class_t ;

struct StaticTerm ;
struct FlatTerm ;
struct ShortTerm ;
struct ObjectTerm ;

}

#endif // __rf_types_hh__
