#include "rf_word.ih"
#include "rf_expr.ih"
#include "rf_types.ih"

namespace rftype
{

using namespace rfrt ;

void Word::ctor (Word* _to, Word const* _from)
{
  _to->data1 = _from->data1;
  _to->ptr_data2 = _from->ptr_data2;
  (static_cast<Header*>(_to->ptr_data2))->ref_count++;
}

void Word::dtor (Word* _word)
{
  Header* h = static_cast<Header*>(_word->ptr_data2);
  if (--(h->ref_count) == 0) allocator->deallocate(h);
}

bool Word::eq (Word const* _word1, Word const* _word2)
{
  Header* h1 = static_cast<Header*>(_word1->ptr_data2);
  Header* h2 = static_cast<Header*>(_word2->ptr_data2);
  if (h1 == h2) return true;
  else {
    if (hash(_word1) != hash(_word2)) return false;
    else if (h1->length != h2->length) return false;
    else if (wmemcmp(h1->content, h2->content, h1->length) == 0) return true;
    else return false;
  }
}

pxx::WString Word::to_string (Word const* _word)
{
  Header* h = static_cast<Header*>(_word->ptr_data2);
  return pxx::WString(h->content, h->length);
}

Expr Word::create_expr (wchar_t const* _wstr)
 {
  Expr e = Term::create_expr(1);
  Term* p = e.get_first();
  new(p) Word(_wstr);
  return e;
}

Expr Word::create_expr (char const* _str, char const* _locale /* = null */)
{
  Expr e = Term::create_expr(1);
  Term* p = e.get_first();
  new(p) Word(_str, _locale);
  return e;
}

Expr Word::create_expr (Expr const& _expr) {
  Term* p = _expr.get_first();

  pxx::WString str;

  for (; p < _expr.get_last(); p++){
    str = str + pxx::WString(*p);
  }

  Expr e = Term::create_expr(1);
  Term* t = e.get_first();
  new(t) Word(str.get_data());
  return e;  
}


int Word::compare (Word const* _word1, Word const* _word2)
{
  Header* h1 = static_cast<Header*>(_word1->ptr_data2);
  Header* h2 = static_cast<Header*>(_word2->ptr_data2);
  if (h1 == h2) return 0;
  else {
    int r = wmemcmp(h1->content, h2->content, pxx_min(h1->length, h2->length));
    if (r != 0) return r;
    else {
      if (h1->length == h2->length) return 0;
      else return h1->length > h2->length ? 1 : -1;
    }
  }
}

uint32_t Word::hash (Word const* _word)
{
  Header* h = static_cast<Header*>(_word->ptr_data2);
  if (h->hash == 0xffffffff) {
    // Should never occur, because we always compute hash for words
    h->hash = compute_hash(h->content, h->length * sizeof(wchar_t));
  }
  return h->hash;
}

short_type_funcs_t Word::funcs = {
  0,
  (void (*)(Term*, Term const*))(Word::ctor),
  (void (*)(Term*))(Word::dtor),
  (bool (*)(Term const*, Term const*))(Word::eq),
  (int (*)(Term const*, Term const*))(Word::compare),
  (uint32_t (*)(Term const*))(Word::hash),
  (pxx::WString (*)(Term const*))(Word::to_string),
  null
};

}
