#ifndef __rf_word_hh__
#define __rf_word_hh__

#include "rf_types.hh"
#include "rf_term.hh"
#include "rf_expr.hh"

namespace rftype
{

using namespace rfrt ;

class Word :
  public Term
{

  struct Header
  {
    size_t ref_count ;
    size_t length ;
    uint32_t hash ;
    wchar_t content[] ;
  };

public:
  inline Word (wchar_t const* _wstr, size_t _len) ;
  ///
  /// Constructor from a string of wide characters
  inline Word (wchar_t const* _wstr) ;
  ///
  inline Word (pxx::WString const& _wstr) ;
  ///
  /// Constructor from a locale-dependent string
  inline Word (char const* _str, char const* _locale = null) ;
  ///
  /// Copy constructor
  inline Word (Word const& _word) ;
  ///
  /// Destructor

  inline ~Word () ;
  ///
  /// Assignment operator
  inline Word& operator = (Word const& _word) ;

  inline size_t get_len () const;

  static Expr create_expr (wchar_t const* _wstr) ;

  static Expr create_expr (char const* _str, char const* _locale = null) ;

  static Expr create_expr (Expr const& _expr) ;

  static void ctor (Word* _to, Word const* _from) ;
  static void dtor (Word* _word) ;
  static bool eq (Word const* _word1, Word const* _word2) ;
  static int compare (Word const* _word1, Word const* _word2) ;
  static uint32_t hash (Word const* _word) ;
  static pxx::WString to_string (Word const* _word) ;
  static short_type_funcs_t funcs ;
  static TypeRegister reg ;

  ///
  /// Term class for template instantiations
  static const term_class_t term_class = short_term;
};



}
#if 0
  typedef enum {plus, minus, positive_number, 
                not_a_number, space_number} is_number_res;
  
  inline enum is_number_res is_number () const;

  inline int has_spaces () const ;
  class NumIterator 
  {
  private:
    const wchar_t* iter;
    const wchar_t* bound;
  public:
    inline NumIterator(Word const& _word) ;
    inline NumIterator& operator ++(int) ;
    inline NumIterator const& operator =(Word const& _word) ;
    inline bool in_bounds () const;
    inline const char operator*() const;
  }; 

#endif

#endif // __rf_word_hh__

